#!/usr/bin/perl
#
# cons-test [-x cons] [test_script ...]
#
#	Run the cons regression test suite.
#

require "getopts.pl";

($My_Name = $0) =~ s(.*/)();	# basename
$Usage="Usage:  $My_Name [-x cons] [test_script ...]";

&Getopts('x:') || die "$Usage\n";

if (defined($opt_x)) {
	$Cons = $ENV{'PROG'} = $opt_x;
	if (! -f $Cons) {
		print STDERR "$My_Name:  The specified '$Cons' script does not exist.\n";
		print STDERR "\tCreate it, or use -x to specify some other executable script.\n";
		print STDERR "$Usage\n";
		exit 1
	} elsif (! -x $Cons) {
		print STDERR "$My_Name:  The specified '$Cons' script is not executable.\n";
		print STDERR "\tSet execute permissions on it,\n\tor use -x to specify some other executable script.\n";
		print STDERR "$Usage\n";
		exit 1
	}
} else {
	$Cons = 'cons';
	if (! -f $Cons) {
		print STDERR "$My_Name:  There is no executable '$Cons' script in the current directory.\n";
		print STDERR "\tCreate one, or use -x to specify some other executable script.\n";
		print STDERR "$Usage\n";
		exit 1;
	} elsif (! -x $Cons) {
		print STDERR "$My_Name:  The '$Cons' script in the local directory is not executable.\n";
		print STDERR "\tSet execute permissions on it,\n\tor use -x to specify some other executable script.\n";
		print STDERR "$Usage\n";
		exit 1;
	}
}

my $pass = 0;
my @fail;

sub report {
	my($sofar) = @_;
	if (@fail == 0) {
		print "$My_Name:  '$Cons' passed all $pass tests$sofar.\n";
	} else {
		printf "$My_Name:  '$Cons' passed $pass tests, failed %d$sofar:\n", scalar @fail;
		print "\t\t", join("\n\t\t", @fail), "\n";
	}
}

my $child_pid;

sub handler {
	my($sig) = @_;
	waitpid($child_pid, 0);
	print "$My_Name:  Caught SIG$sig; exiting.\n";
	print "\n";
	&report(' so far');
	exit (1);
}

$SIG{'HUP'} = \&handler;
$SIG{'INT'} = \&handler;
$SIG{'QUIT'} = \&handler;
$SIG{'TERM'} = \&handler;

if (!@ARGV) {
	opendir(DIR, '.') || die "$My_Name: cannot open current directory: $!\n";
	@ARGV = sort grep(/^t[0-9]+a.sh$/, readdir(DIR));
	closedir(DIR);
}

$| = 1;		# flush print

foreach $test (@ARGV) {
	my($cmd) = "sh $test";
	print "$My_Name:  $cmd\n";
	$child_pid = fork;
	if ($child_pid == 0) {
		exec $cmd;
	}
	waitpid($child_pid, 0);
	$exit = $?;
	if ($exit) {
		push(@fail, $test);
	} else {
		$pass++;
	}
}

print "\n";
&report();

exit @fail;
