#!/bin/sh
#
#	Compile a single executable from three .c files in a source
#	subdirectory.  One .c file has a conditional #include for
#	the selected OS.  Compilation takes place in two separate
#	build subdirectories with separate Conscript files that
#	establish 'Link' to the source subdirectory and separate
#	build environments.  Both Conscript builds are invoked from
#	the same invocation of Cons.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multiple-environment variant builds]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir src
if test $? -ne 0 ; then no_result ; fi
mkdir build
if test $? -ne 0 ; then no_result ; fi
mkdir build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir build/bar
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
Build qw(
	build/foo/Conscript
	build/bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >build/foo/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DFOO',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >build/bar/Conscript
Link 'src' => '#src';

$env = new cons(
	CFLAGS	=> '-DBAR',
);

Export qw( env );

Build qw(
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/Conscript
Import qw( env );
Program $env 'xxx', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/a.c
a()
{
	printf("In function a().\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/b.c
b()
{
	printf("In function b().\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/main.c
main()
{
	a();
	b();
#ifdef	FOO
	printf("FOO!\n");
#endif
#ifdef	BAR
	printf("BAR!\n");
#endif
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . > cons.1.out 2>&1
if test $? -ne 0 ; then fail ; fi
build/foo/src/xxx > foo.out
if test $? -ne 0 ; then fail ; fi
diff - foo.out <<'_EOF_'
In function a().
In function b().
FOO!
_EOF_
if test $? -ne 0 ; then fail ; fi
build/bar/src/xxx > bar.out
if test $? -ne 0 ; then fail ; fi
diff - bar.out <<'_EOF_'
In function a().
In function b().
BAR!
_EOF_
if test $? -ne 0 ; then fail ; fi

#
pass
