#!/bin/sh
#
#	Create two executables from .c files in the local directory,
#	and two in a subdirectory.  Remove all derived files (-r).
#	Remove the first subdirectory .c file.  Build the subdirectory;
#	see it fail; make sure neither .o or executable was built.
#	Remove the first local directory .c file.  Build the world.
#	Make sure no .o files nor executables were built.  Build
#	-k the subdirectory.  Make sure the second "masked"
#	subdirectory executable was built.    Build -k the world.
#	Make sure the second "masked" local directory executable
#	was built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-k]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir subdir
if test $? -ne 0 ; then no_result ; fi

#
cat <<'_EOF_' >Construct
$env = new cons;
Program $env 'a', 'a.c';
Program $env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
main()
{
	printf("a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
main()
{
	printf("b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/Conscript
$env = new cons;
Program $env 'c', 'c.c';
Program $env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/c.c
main()
{
	printf("subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/d.c
main()
{
	printf("subdir/d.c\n");
	exit (0);
}
_EOF_

$prog . >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 ; fi
./a >a.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - a.1.out <<'_EOF_'
a.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
./b >b.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - b.1.out <<'_EOF_'
b.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/c >c.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - c.1.out <<'_EOF_'
subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
subdir/d >d.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - d.1.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog . >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - cons.2.out <<'_EOF_'
cons: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

$prog -r . >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.out ; fi

rm -f subdir/c.c
if test $? -ne 0 ; then no_result ; fi

$prog subdir >cons.4.out 2>cons.4.err
if test $? -eq 0 ; then fail 4 cons.4.out ; fi	# expect failure, -eq 0
if test -e subdir/c.o ; then fail 4 ; fi
if test -e subdir/c ; then fail 4 ; fi
if test -e subdir/d.o ; then fail 4 ; fi
if test -e subdir/d ; then fail 4 ; fi

rm -f a.c
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.5.out 2>cons.5.err
if test $? -eq 0 ; then fail 5 cons.5.out ; fi	# expect failure, -eq 0
if test -e a.o ; then fail 5 ; fi
if test -e a ; then fail 5 ; fi
if test -e b.o ; then fail 5 ; fi
if test -e b ; then fail 5 ; fi
if test -e subdir/c.o ; then fail 5 ; fi
if test -e subdir/c ; then fail 5 ; fi
if test -e subdir/d.o ; then fail 5 ; fi
if test -e subdir/d ; then fail 5 ; fi

$prog -k subdir >cons.6.out 2>cons.6.err
if test $? -eq 0 ; then fail 6 cons.6.out ; fi	# expect failure, -eq 0
if test -e subdir/c.o ; then fail 6 ; fi
if test -e subdir/c ; then fail 6 ; fi
subdir/d >d.6.out
if test $? -ne 0 ; then fail 6 ; fi
diff - d.6.out <<'_EOF_'
subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 6 ; fi

$prog -k . >cons.7.out 2>cons.7.err
if test $? -eq 0 ; then fail 7 cons.7.out ; fi	# expect failure, -eq 0
if test -e a.o ; then fail 7 ; fi
if test -e a ; then fail 7 ; fi
./b >b.7.out
if test $? -ne 0 ; then fail 7 ; fi
diff - b.7.out <<'_EOF_'
b.c
_EOF_
if test $? -ne 0 ; then fail 7 ; fi

#
pass
